function result = ADMM_BI_batch_V_correct_CayleySGD_huber(Y,Vs, r,c,Paras)
% Created on 08-01, update on 08-12
% ADMM with cayleySGD line search
result = struct;[N,P] = size(Y);
%% Parameters
if isfield(Paras, "stop_cri")
    stop_cri = Paras.stop_cri;
else
    stop_cri = 1e-5;
end

if isfield(Paras, "N_step")
    N_step = Paras.N_step;
else
    N_step = 100;
end

if isfield(Paras, "g_init")
    g_init = Paras.g_init;
else
    g_init = ones(N,1);
end
if isfield(Paras, "V_init")
    V_init = Paras.V_init;
else
    V_init = Vs;
end
if isfield(Paras, "mu")
    mu = Paras.mu;
else
    mu = 1e-8;
end
%% Paras
% Paras = struct; 
Paras.alpha0 = 1; Paras.beta = 0.5; Paras.stop_inner = 1e-5;
alpha0 = Paras.alpha0; beta = Paras.beta; stop_inner = Paras.stop_inner;
%% Updates
g = g_init; 
%V = Vs;
V = V_init;
g_old = g; Vold = V; obj_g = zeros(N_step,1); obj_V = zeros(N_step,1); 
for n_step = 1:N_step
    update_g = BI_solver_g(g,Y,V,r,c,Paras);
    g = update_g.g;
    obj_g(n_step) = norm(V*diag(g)*V'*Y,1)+Paras.rho_s/2*norm(Vs-V,'fro')^2;
    V = CayleySGD_linesearch_huber(V,Vs,g,Y,Paras.rho_s,mu,alpha0,beta,50);
    obj_V(n_step) = huber_loss(V*diag(g)*V'*Y,mu)+Paras.rho_s/2*norm(Vs-V,'fro')^2;
    RE_v = norm(g - g_old)/(1e-10 + norm(g_old));
    RE_V = norm(V(:) - Vold(:))/(1e-10 + norm(V(:)));  
    if RE_v^2 + RE_V^2 <= stop_cri^2
        fprintf(['Break at ', num2str(n_step), ', with RE_g = ', num2str(RE_v), ', RE_V = ', num2str(RE_V)]);
        break;
    else
        g_old = g;Vold = V;
    end        
    if mod(n_step,floor(N_step/50)) == 0
        % figure(1);imagesc(V*diag(g)*V');
        fprintf(['(Cayley V solver v2) Re_V = ', num2str(RE_V),', RE_g = ', num2str(RE_v),'\n']);
    end
    if n_step == N_step
        fprintf('Max iteration reached! \n');
    end
end
if n_step < N_step
    obj_g(n_step:N_step) = ones(N_step-n_step+1,1)*obj_g(n_step-1);
    obj_V(n_step:N_step) = ones(N_step-n_step+1,1)*obj_V(n_step-1);
end
result.g = g;
result.V = V; Z = kr(Y'*V,V);
result.x = Z*g;
% result.Lambda = update_V.Lambda;
result.mu = update_g.mu;
result.ite = n_step;
result.RE_v = RE_v;
result.obj_g = obj_g;
result.obj_V = obj_V;
