function X = X_generate(paras)
%% Generating full rank X
symmetry = paras.symmetry;
switch symmetry
    case 0
        S = paras.S;
        P = paras.P;
        N = paras.N;
        X = randn(S, P); % Generate the values in the support of the P signals
        X = X./repmat(sqrt(sum(abs(X).^2)), S, 1); % Normalize the signals
        X = [X; zeros(N-S, P)]; % Pad zeros to generate the full signals
    case 1
        S = paras.S;
        P = paras.P;
        N = paras.N;
        mu = paras.mu;
        sigma = paras.sigma;
    R = abs(random('Normal',mu,sigma,[S,P]));
    sign = (random('bino',1,0.5,[S,P])-0.5)*2;
    X = R.*sign;
    X = [X; zeros(N-S, P)]; % Pad zeros to generate the full signals 
    for p = 1:P
        temp_X = X(:,p);
        Randindex =  randperm(N);
        X(:,p) = temp_X(Randindex);
    end
    case 2 % R: uniform
        S = paras.S;
        P = paras.P;
        N = paras.N;        
    R = rand(S,P);
    sign = (random('bino',1,0.5,[S,P])-0.5)*2;
    X = R.*sign;
    X = [X; zeros(N-S, P)]; % Pad zeros to generate the full signals 
    for p = 1:P
        temp_X = X(:,p);
        Randindex =  randperm(N);
        X(:,p) = temp_X(Randindex);
    end    
    case 3 % R: binary
        S = paras.S;
        P = paras.P;
        N = paras.N;
    R = randi(1,S,P);
    sign = (random('bino',1,0.5,[S,P])-0.5)*2;
    X = R.*sign;
    X = [X; zeros(N-S, P)]; % Pad zeros to generate the full signals 
    for p = 1:P
        temp_X = X(:,p);
        Randindex =  randperm(N);
        X(:,p) = temp_X(Randindex);
    end      
    case 4 % R: uniform, change K support when p -> p+1
        S = paras.S;
        P = paras.P;
        N = paras.N;
        K = paras.K;
    R = rand(N,P);
    sign = (random('bino',1,0.5,[N,P])-0.5)*2;
    X = R.*sign;
    indexs = randperm(N);
    % Random shuffle first S index of the support
    % Then random shuffle from the (S-K+1) to N -th indexs
    indexs(1:S) = indexs(randperm(S));
    temp = indexs(S+1-K:N); 
    indexs(S+1-K:N) = temp(randperm(N-S+K));
    X(indexs(S+1:N),1) = 0;
    for p = 1:P
        X(indexs(S+1:N),p) = 0;
        indexs(1:S) = indexs(randperm(S));
        temp = indexs(S+1-K:N); 
        indexs(S+1-K:N) = temp(randperm(N-S+K));
    end        
    case 5 % R: uniform, change K support when p -> p+1
        S = paras.S;
        P = paras.P;
        N = paras.N;
        K = paras.K;
        beta = paras.beta;
    R = rand(N,P);
    sign = (random('bino',1,0.5,[N,P])-0.5)*2;
    X = R.*sign;
    indexs = randperm(N);
    % Random shuffle first S index of the support
    % Then random shuffle from the (S-K+1) to N -th indexs
    indexs(1:S) = indexs(randperm(S));
    temp = indexs(S+1-K:N); 
    indexs(S+1-K:N) = temp(randperm(N-S+K));
    X(indexs(S+1:N),1) = 0;
    for p = 2:P
        X(indexs(S+1:N),p) = 0;
        x1 = X(:,p-1); x2 = X(:,p);ct = (norm(x1-x2)^2)*(p^beta);
        alpha = rescale_x(x1,x2,ct);
        X(:,p) = x2*alpha;        
        indexs(1:S) = indexs(randperm(S));
        temp = indexs(S+1-K:N); 
        indexs(S+1-K:N) = temp(randperm(N-S+K));
    end        
    case 6 % R: uniform, change K support when p -> p+1
        S = paras.S;
        P = paras.P;
        N = paras.N;
        K = paras.K;
        beta = paras.beta;
    R = rand(N,P);
    sign = (random('bino',1,0.5,[N,P])-0.5)*2;
    X = R.*sign;
    indexs = randperm(N);
    % Random shuffle first S index of the support
    % Then random shuffle from the (S-K+1) to N -th indexs
    indexs(1:S) = indexs(randperm(S));
    temp = indexs(S+1-K:N); 
    indexs(S+1-K:N) = temp(randperm(N-S+K));
    X(indexs(S+1:N),1) = 0;
%     X(:,1) = X(:,1)*N/norm(X(:,1));
    for p = 2:P     
        indexs(1:S) = indexs(randperm(S));
        temp = indexs(S+1-K:N); 
        indexs(S+1-K:N) = temp(randperm(N-S+K));
        X(indexs(S+1:N),p) = 0;
        x1 = X(:,p-1); x1(indexs(S+1:N)) = 0; x2 = X(:,p);ct = (p^beta);
%         X(:,p) = (1-ct)*x1+ct*x2; 
        X(:,p) = ((1-ct)*X(:,p-1) + ct*x2).*(abs(x2)>=1e-5);
        X(:,p) = X(:,p)/norm(X(:,p))*norm(x2);
%         X(:,p) = (1-ct)*x1+ct*x2;
%         X(:,p) = x2*alpha;           
    end            
end